/*
 * @copyright   © Copyright 2006-2012, epages GmbH, All Rights Reserved.
 *
 * @module      de_epages.externalcontent.ui.gadgets
 *
 * @revision    $$
 */

jQuery.plugin('de_epages.externalcontent.gadgets', {
  plugin: ['de_epages.core', 'ep.dict', 'ep.core', 'ep.ui.dialog', 'backbone', 'jQuery.tmpl', 'jQuery.json', 'ep.ajax', 'jQuery.metaparse'],
  dict: ['ep.dict', 'de_epages.externalcontent.gadgets'],
  tmpl: ['de_epages.externalcontent.overview']
}, function ($, undefined) {

/*
 * @dictionary ep.dict
 *
 * @translation
 * {Close}
 * {Apply}
 * {Overview}
 * {Action}
 * {EMail}
 * {Title}
 * {ContactPerson}
 * {Address}
 * {BackgroundColor}
 * {Overview}
 * {UserName}
 * {URL}
 * {Size}
 * {SearchTerm}
 * {AllEntries}
 *
 * @dictionary de_epages.externalcontent.gadgets
 * {InsertNewGadget}
 * {Required}
 * {Optional}
 * {rss}
 * {Feed}
 * {NumberOfEntries}
 * {OnlySnippets}
 * {OnlyHeadlines}
 * {ShowImages}
 * {Layout}
 * {Default}
 * {Box}
 * {RSSGadgetDescription}
 * {maps}
 * {ShowControls}
 * {FixedView}
 * {GoogleMapsExplanation}
 * {RoutePlanner}
 * {ContactData}
 * {LargeNavigation}
 * {MapRight}
 * {translation}
 * {Service}
 * {externalvideo}
 * {ExternalVideoExplanation}
 * {SupportedVideoPortals}
 * {VideoURLorEmbed}
 * {facebook}
 * {FacebookButtonExplanation}
 * {ShowFaces}
 * {Like}
 * {Recommend}
 * {Default}
 * {NoCounter}
 * {WithCounter}
 * {ButtonCount}
 * {ButtonBox}
 * {ShowFacesInfo}
 * {FacebookProfilePage}
 * {ColorScheme}
 * {Light}
 * {Dark}
 * {ShowStream}
 * {ShowHeader}
 * {AutoHeight}
 * {SubHeading}
 * {twitter}
 * {TwitterButtonExplanation}
 * {Text}
 * {TitleOfCurrentPage}
 * {URLofCurrentPage}
 * {googleplus}
 * {Small}
 * {Medium}
 * {Tall}
 * {otherbuttons}
 */

  var globalDict = ep.dict('ep.dict');
  var moduleDict = ep.dict('de_epages.externalcontent.gadgets');

  //
  //	The Basics.
  //

  // Constants of a gadget as it will be generated
  // and inserted into the DOM.
  var gadgetDialogInstance,
      gNamespace = 'de_epages.externalcontent',


      // Default gadget data.
       // With *requireJS* this could easily be moved to a
       // external config.js file or something of that fashion.
      gDefaults = {
      'contentslider': {
        'effect': 'fade',
        'arrows': false,
        'shadow': false,
        'border': 'transparent',
        'automatic': true,
        'displaytime': 3,
        'slides': []
      },
      'maps': [{
        'type': 'google',
        'width': 500,
        'height': 300,
        'zoom': 5,
        'title': (ep.config.shopCompany || '') + (ep.config.shopCompany ? ', ' : '') + (ep.config.shopAddress || ''),
        'controls': true,
        'fixedView': false
      },
      {
        'type': 'bing',
        'width': 500,
        'height': 300,
        'zoom': 5,
        'title': (ep.config.shopCompany || '') + (ep.config.shopAddress || ''),
        'controls': true,
        'fixedView': false
      },
      {
        'type': 'klicktel',
        'width': 400,
        'height': 500,
        'address': (ep.config.shopAddress || ''),
        'contactPerson': '',
        'email': '',
        'useRoute': false,
        'showAddress': false,
        'largeNavigation': false,
        'layoutLR': false
      }],
      rss: {
        url: '',
        numberOfEntries: 10,
        onlyHeadlines: false,
        onlySnippets: false,
        showImages: true,
        layout: 'default'
      },
      googleplus: {
        layout: 'standard',
        type: 'bubble'
      },
      translation: {
        service: 'google'
      },
      otherbuttons: {
        services: ['digg', 'delicious', 'stumbleupon', 'technorati', 'mrwong']
      },
      facebook: [{
        type: 'button',
        action: 'like',
        layout: 'standard',
        showFaces: true
      },
      {
        type: 'box',
        url: '',
        color: 'light',
        width: 290,
        height: '',
        showHeader: false,
        showStream: false,
        showFaces: true
      }],
      twitter: [{
        type: 'share',
        text: '',
        username: '',
        url: '',
        count: 'none'
      },
      {
        type: 'follow',
        username: ''
      },
      {
        type: 'profile',
        username: '',
        width: 230,
        height: 250
      },
      {
        type: 'search',
        searchterm: '',
        title: '',
        subject: ''
      }],
      externalvideo: {
        width: 560,
        height: 315
      }
      },
      gadgetData = {};
  // We will store gadget data from server here.
  //
  // Views
  //
  // The »App«.
  // TODO: Comments about functionality.
  var GadgetsDialog = (function ($, Backbone) {

    var view = Backbone.View.extend({
      tagName: 'div',
      render: function () {
        if (this.content) {
          this.$el.append(this.content.render().$el);
        }
        return this;
      },
      closeDialog: function () {
        if (this.content) {
          this.content.remove();
          this.content = null;
        }
        this.dialogNode.uiDialog('close');
      },
      openDialog: function (options) {
        var self = this;
        if (options.gadget) {
          // Open configuration view of *options.gadget* gadget.
          self.options = options;
          self.select(options.gadget);
        }
        else {
          // Callback may have changed.
          self.options = options;
          // Open *OverView* and let user *select* a gadgets to insert.
          if (self.content && self.content.$el) {
            self.content.$el.remove();
          }
          self.content = new OverView(gadgetData);
          self.render();
          self.content.on('select', self.select, self);
          self.dialogNode.uiDialog('option', 'buttons', [{
            text: globalDict.translate('{Close}'),
            click: $.proxy(self.closeDialog, self)
          }]);
        }
        this.dialogNode.uiDialog('option', 'title', moduleDict.translate('{InsertNewGadget}'));
        this.dialogNode.uiDialog('open');

      },
      apply: function () {
        // *this.content.onApply* can still stop the process.
        var doApply = true;
        if (this.content && typeof this.content.onApply === 'function') {
          doApply = this.content.onApply();
        }

        if (doApply) {
          if (this.options.callback) {
            this.options.callback({
              'options': this.content.gOptions,
              'alias': this.content.alias,
              'cartridge': 'externalcontent'
            });
          }
          this.closeDialog();
        }

      },
      select: function (gadget) {
        var buttons = [];
        // Switch to configuration View.
        // Setup "Apply" button.
        // Currently there are no *predefined* gadgets,
        // the *else* part could be used for a later implementation.
        if ( /*!gadget.isPredefined &&*/ GadgetView[gadget.alias]) {
          // Remove old *content*, if any.
          if (this.content && this.content.$el) {
            this.content.$el.remove();
          }
          // Open *gadget.options* gadget configuration.
          if (gadget.options) {
            this.content = new GadgetView[gadget.alias](gadget.options);
          }
          else {
            this.content = new GadgetView[gadget.alias]();
          }
          this.render();

          buttons.push({
            text: globalDict.translate('{Apply}'),
            click: $.proxy(this.apply, this)
          });

          buttons.push({
          	text: globalDict.translate('{Cancel}'),
            click: $.proxy(this.closeDialog, this)
          });

          this.dialogNode.uiDialog('option', {
            'buttons': buttons,
            'title': moduleDict.translate(gadget.alias)
          });
				}
      },
      initialize: function () {
        var self = this,
            parentEp = ep;
        try {
          parentEp = top.ep;
        }
        catch (error) {}
        var initDialog = function () {
          // use top object to open dialog always in top frame (because preview iframe is to small for dialog)
          self.dialogNode = parentEp(self.render().$el).uiDialog({
          	stack: false, // Quick fix to have colorpicker in contentslider view on top even after clicking somewhere else on the dialog.
            width: 900,
            height: 600,
            modal: false,
            buttons: [{
              text: globalDict.translate('{Close}'),
              click: $.proxy(self.closeDialog, self)
            }],
            title: moduleDict.translate('{InsertNewGadget}')
          });
        };
        if (self.options.gadget) {
          initDialog();
          self.select(self.options.gadget);
          //load gadgetData from Server
          ep.ajax({
            data: {
              'ViewAction': 'JSONGetGadgetInformation',
              'ObjectID': ep.config.siteId
            },
            type: 'POST',
            dataType: 'json'
          }).done(function (jsonData) {
            gadgetData = jsonData;
          });
        }
        else {
          //load gadgetData from Server
          ep.ajax({
            data: {
              'ViewAction': 'JSONGetGadgetInformation',
              'ObjectID': ep.config.siteId
            },
            type: 'POST',
            dataType: 'json'
          }).done(function (jsonData) {
            gadgetData = jsonData;
            self.content = new OverView(gadgetData);
            self.content.on('select', self.select, self);
            initDialog();
          });
        }
      }
    });

    return view;
  })(jQuery, Backbone);

  var OverView = (function ($, Backbone) {
    // 'use strict';
    var view = Backbone.View.extend({
      tagName: 'div',
      template: 'overview',
      events: {
        'change div div ul li input': 'select'
      },
      render: function () {
        this.$el.append(
        $.tmpl(gNamespace + '.' + this.template, this.options).dictParse('ep.dict', true).dictParse('de_epages.externalcontent.gadgets', true));
        this.$el.find('.ep-js').metaparse();
        return this;
      },
      select: function (e) {
        var $target = $(e.target);
        if ($target.is('img')) {
          $target = $target.parent();
        }
        this.trigger('select', {
          alias: $target.data('alias').toLowerCase(),
          isPredefined: $target.data('isPredefined') ? true : false
        }); // See this.template.
      }
    });

    return view;
  })(jQuery, Backbone);

  var GadgetViewPrototype = (function ($, Backbone, de_epages) {
    // 'use strict';
    var view = Backbone.View.extend({
      tagName: 'div',
      setOption: function (key, value, refreshPreview) {
        var self = this;
        self.gOptions[key] = value;
        if (refreshPreview) {
          self.renderPreview();
        }
        return self;
      },
      setOptions: function (options) {
        var self = this,
            key;
        for (key in options) {
          if (options.hasOwnProperty(key)) {
            self.setOption(key, options[key]);
          }
        }
        return self;
      },
      renderPreview: function () {
        var self = this,
            preview = self.elements.preview,
            parent = preview.parent(),
            gadgetPath = 'externalcontentGadget' + self.alias.charAt(0).toUpperCase() + self.alias.slice(1);
        preview.remove();
        preview = self.elements.preview = de_epages('<div class="PreviewNode">').appendTo(parent);
        preview[gadgetPath](self.gOptions);
      },
      onSelect: function (e, noPreview) {
        var target = e.target,
            selected = $(target).find(':selected');
        this.setOption(target.name, selected.val(), noPreview ? false : true);
      },
      // Update options on input.
      onInput: function (e, noPreview) {
        var target = e.target,
            $target = $(target);
        if (target.type === 'checkbox') {
          var siblings = $target.closest('fieldset');
          if (siblings.length === 0) {
            siblings = $target.closest('form');
          }
          siblings = siblings.find(':checkbox[name="' + target.name + '"]');
          if (siblings.length > 1) {
            var valueArray = [];
            siblings.filter(':checked').each(function () {
              valueArray.push($(this).val());
            });
            this.setOption(target.name, valueArray, noPreview ? false : true);
          }
          else {
            this.setOption(target.name, target.checked, noPreview ? false : true);
          }
        }
        else { //type text
          this.setOption(target.name, target.value, noPreview ? false : true);
        }
      },
      onSubmit: function () {
        return false;
      },
      onRadio: function (e) {
        var target = e.target,
            self = this;
        if (target.value !== self.gOptions[target.name]) {
          // Set *self.gOptions* according to newly chosen type (i.e. *target.value*) and
          // show fieldset with name *target.value*, hide the others.
          self.gOptions = {};
          self.gOptions[target.name] = target.value;
          self.$el.find('fieldset').not('[name="' + target.name + '"], [name="' + target.value + '"]').attr('disabled', true).addClass('HideElement');
          self.$el.find('fieldset[name="' + target.value + '"]').attr('disabled', false).removeClass('HideElement');
          // Set remaining *self.gOptions* according to fieldset shown.
          self.$el.find('fieldset[name="' + target.value + '"] :input').trigger('change', true);
          self.renderPreview();
        }
      },
      onApply: function () {
        // TODO VALIDATION
        // https://jira.epages.de/browse/EPG-26271
        return true;
      },
      getAPI: function () {
        // Overwritten by the individual gadget views
      },
      // Init options of the gadget.
      initialize: function () {
        var self = this,
            gotNoOptions = $.isEmptyObject(self.options);
        self.differTypes = $.isArray(gDefaults[self.alias]);
        self.gDefaults = $.extend(true, self.differTypes ? [] : {}, gDefaults[self.alias]);
        self.plugin = gNamespace + '.gadget.' + self.alias;
        self.template = gNamespace + '.' + self.alias + 'view';
        self.getAPI();
        if (self.differTypes) {
          if (gotNoOptions) {
            self.gOptions = self.gDefaults[0];
          }
          else {
            self.gOptions = self.options;
          }
        }
        else {
          self.gOptions = gotNoOptions ? self.gDefaults : self.options;
        }
        self.verifyUrl = null;
      },
      render: function () {
        var self = this,
            types = [],
            i;

        if (self.differTypes) {
          // Merge *self.gOptions* in *types* for templating. (See below.)
          for (i = 0; i < self.gDefaults.length; i++) {
            var defaults = self.gDefaults[i];
            types.push(defaults.type === self.gOptions.type ? self.gOptions : defaults);
          }
        }

        $.ready({
          plugin: [self.plugin, 'ep.ui.input', 'ep.ui.validate'],
          tmpl: self.template
        }, function () {
          var renderedTemplate = $.tmpl(self.template, self.differTypes ? {
            'types': types,
            'gOptions': self.gOptions
          } : self.gOptions).dictParse('ep.dict', true).dictParse('de_epages.externalcontent.gadgets', true);
          renderedTemplate.find('.Tooltip').each(function () {
            ep(this).uiTooltip({
              interactive: true
            });
          });
          renderedTemplate.find(':input').each(function () {
            if (this.placeholder) {
              this.placeholder = globalDict.translate(this.placeholder);
              this.placeholder = moduleDict.translate(this.placeholder);
            }
            if (this.required || this.type === 'number') {
              //ep(this).uiValidate();
            }
            else {
              ep(this).uiInput();
            }
          });
          self.$el.append(renderedTemplate);
          self.elements = renderedTemplate.tmplItem('elements'); // preview element z.B.
          self.elements.preview = de_epages(self.elements.preview);
          self.renderPreview(); // TODO Preview View?
        });
        return this;
      }
    }, {
      events: {
        'submit form': 'onSubmit',
        'change input[type!="radio"]': 'onInput',
        'change input[type="radio"]': 'onRadio',
        'change select': 'onSelect'
      }
    });
    return view;
  })(jQuery, Backbone, de_epages);

  //
  // Define the individual gadget (configuration) views.
  //
  var GadgetView = {};

  // Standard views.
  var standardGadgets = ['otherbuttons', 'facebook', 'twitter'];
  $.each(standardGadgets, function (index, alias) {
    GadgetView[alias] = GadgetViewPrototype.extend({
      alias: alias,
      events: GadgetViewPrototype.events
    });
  });

  // Special (more individual) views.
  $.extend(GadgetView, {

    contentslider: (function ($, Backbone, GadgetViewPrototype) {
      'use strict';
      var view = GadgetViewPrototype.extend({
        alias: 'contentslider',
        cropImageOptions: {},
        cropDialogOptions: {
        	width: 1070,
          minHeight: 570,
          modal: true,
          //draggable: false,
          title: moduleDict.translate('{CropImage}')
        },
        defaultSize: {
          width: 1050,
          height: 350
        },
        events: $.extend({}, GadgetViewPrototype.events, {
          //'colorSelect button': 'onColorSelect',
          'click .ep-sprite.ico_s_delete': 'deleteSlide',
          'click .ep-sprite.object_ico_s_link': 'linkSlide',
          'click .ep-sprite.ico_s_edit': 'editSlide',
          'slide form': 'onSlide'
        }),
        onColorSelect: function (event, color) {
          this.elements.borderColorpicker.css('background', color);
          if (color === 'transparent') {
          	this.elements.borderColorpicker.css({
          		'background-image': 'url(' + ep.config.iconsRoot + '/img_transparent.gif)',
          		'background-size' : '100%'
          	});
          }
          this.setOption('border', color, true);
        },
        onSlide: function (event, ui) {
          this.setOption('displaytime', ui.value, true);
        },
        onSortstart: function (event, ui) {
          this.sortstartedOnPosition = ui.item.prevAll().length;
        },
        onSortupdate: function (event, ui) {
          var slides = this.gOptions.slides;

          // Reorder *slides*.
          slides.splice(ui.item.prevAll().length, 0, slides.splice(this.sortstartedOnPosition, 1)[0]);
          this.setOption('slides', slides, true);
        },
        deleteSlide: function (event) {
          var $li = $(event.target).parent().parent(),
              slides = this.gOptions.slides;

          this.deletedSlidesStack.push(slides.splice($li.prevAll().length, 1)[0].fullpath);
          $li.remove();
          this.elements.slidesList.sortable('refresh');
          this.setOption('slides', slides, true);
        },
        linkSlide: function (event) {
          var $li = $(event.target).parent().parent(),
              index = $li.prevAll().length;

          this.linkpicker.presentationUiLinkpicker('open', {
            url: this.gOptions.slides[index].href || '',
            title: this.gOptions.slides[index].title || '',
            callback: $.proxy(function (url, title, openNewWindow) {
              var slide = this.contentsliderView.gOptions.slides[this.index];
              slide.href = url;
              slide.title = title;
              slide.target = openNewWindow ? '_blank' : '';
              this.contentsliderView.setOption('slides', this.contentsliderView.gOptions.slides, true);
            }, {
              contentsliderView: this,
              index: index
            })
          });
        },
        editSlide: function (event) {
          var $li = $(event.target).parent().parent(),
              index = $li.prevAll().length;

          this.elements.addImage.mediagalleryUiFilemanagerdialog('option', 'contentsliderSlideIndex', index);
          this.elements.addImage.mediagalleryUiFilemanagerdialog('open');
        },
        renderPreview: function () {
          var self = this,
              preview = self.elements.preview,
              parent = preview.parent(),
              gadgetPath = 'externalcontentGadget' + self.alias.charAt(0).toUpperCase() + self.alias.slice(1);
          preview.remove();
          preview = self.elements.preview = de_epages('<div>').insertBefore(self.elements.footer);
          preview[gadgetPath]($.extend(true, {}, self.gOptions, {
            cycleOptions: {
              after: $.proxy(function (currSlideElement, nextSlideElement, options, forwardFlag) {
                this.showLinkInfo($(nextSlideElement).prevAll().length);
              }, this)
            }
          }));
          preview.find('a[href]').each(function () {$(this).removeAttr('href')}); // Links should not work inside the dialog.
          if (self.gOptions.slides.length === 1) {
          	self.showLinkInfo(0);
          }
          if (!self.gOptions.slides.length) {
          	 self.elements.linkInfo.html('');
          }
        },
        showLinkInfo: function (index) {
          var slide = this.gOptions.slides[index];
          if (slide.title || slide.href) {
            this.elements.linkInfo.html(moduleDict.translate('{LinkedWith}') + ': <b>' + slide.title || slide.href + '</b>');
          }
          else {
            this.elements.linkInfo.html('');
          }
        },
        render: function () {
          var self = this;

          $.ready({
            plugin: ['ep.core', 'ep.ui.slider', 'ep.ui.colorpicker', 'jQuery.ui.sortable', 'de_epages.externalcontent.ui.cropslidedialog', 'de_epages.mediagallery.ui.filemanagerdialog', 'de_epages.presentation.ui.linkpicker', self.plugin],
            tmpl: [self.template, 'de_epages.externalcontent.contentsliderview-list']
          }, function () {
            var renderedTemplate = $.tmpl(self.template, self.gOptions);
            self.elements = renderedTemplate.tmplItem('elements');
            // Init *displaytimeSlider*.
            ep(self.elements.displaytimeSlider).uiSlider();
            // Translate text nodes.
            renderedTemplate.dictParse('ep.dict', true).dictParse('de_epages.externalcontent.gadgets', true);
            // Init *uiInput*s and translate placeholder attributes.
            renderedTemplate.find(':input').each(function () {
              if (this.placeholder) {
                this.placeholder = globalDict.translate(this.placeholder);
                this.placeholder = moduleDict.translate(this.placeholder);
              }
              ep(this).uiInput();
            });
            // Translate *title* tags of sprite *spans*.
            renderedTemplate.find('.ep-sprite[title]').each(function () {
                this.title = globalDict.translate(this.title);
                this.title = moduleDict.translate(this.title);
            });
            self.$el.append(renderedTemplate);
            // Sub *de_epages* for preview.
            self.elements.preview = de_epages(self.elements.preview);

            // Quick and dirty fix for preview context problem.
            try {
              window[window.top === window.self ? 'self' : 'top'].jQuery.ready(['de_epages.mediagallery.ui.filemanagerdialog', 'de_epages.externalcontent.ui.cropslidedialog', 'de_epages.presentation.ui.linkpicker', 'ep.ui.colorpicker', 'jQuery.ui.sortable'], function (top$) {
                // Init *sortable* on *slidesList*.
           	 		window[window.top === window.self ? 'self' : 'top'].jQuery(self.elements.slidesList).sortable({
              		cursor: 'move',
									start: $.proxy(self.onSortstart, self),
									update: $.proxy(self.onSortupdate, self)
            		});
                // Init *borderColorpicker*.
                window[window.top === window.self ? 'self' : 'top'].ep(self.elements.borderColorpicker).uiColorpicker({
                  color: self.gOptions.border
                }).on('colorSelect', $.proxy(self.onColorSelect, self));
                // Init *linkpicker*.
                self.linkpicker = window[window.top === window.self ? 'self' : 'top'].de_epages('<div>').presentationUiLinkpicker({
                  autoOpen: false
                });
                // Init *addImage* button.
                self.elements.addImage = window[window.top === window.self ? 'self' : 'top'].de_epages(self.elements.addImage).mediagalleryUiFilemanagerdialog({
                  contentsliderSlideIndex: null,
                  filemanager: {
                    selectable: 'image/'
                  },
//#JSCOVERAGE_IF 0
                  dialog: {
                    close: $.proxy(function (event, ui) {
                      self.elements.addImage.mediagalleryUiFilemanagerdialog('option', 'contentsliderSlideIndex', null);
                    }, self),
                    buttons: [{
                      text: globalDict.translate('{Apply}'),
                      click: $.proxy(function () {
                        var selectedFiles = self.elements.addImage.mediagalleryUiFilemanagerdialog('getSelectedElements');
                        if (selectedFiles.length) {
                          self.cropImage(selectedFiles[0], self.elements.addImage.mediagalleryUiFilemanagerdialog('option', 'contentsliderSlideIndex'));
                        }
                        self.elements.addImage.mediagalleryUiFilemanagerdialog('close');
                      }, self)
                    }]
                  }
                });
                self.renderPreview();
              });
            }
            catch (error) {}
//#JSCOVERAGE_ENDIF
          });
          return this;
        },
        initialize: function () {
          this.gDefaults = $.extend(true, {}, gDefaults[this.alias]);
          this.plugin = gNamespace + '.gadget.' + this.alias;
          this.template = gNamespace + '.' + this.alias + 'view';
          this.gOptions = $.isEmptyObject(this.options) ? this.gDefaults : this.options;
          this.deletedSlidesStack = [];
        },
//#JSCOVERAGE_IF 0
        addImage: function (name, fullpath, index) {
          if (typeof index !== 'undefined' && index !== null) {
            this.deletedSlidesStack.push(this.gOptions.slides[index].fullpath);
            this.gOptions.slides[index].name = name;
            this.gOptions.slides[index].fullpath = fullpath;
            $(this.elements.slidesList.children('li').get(index)).find('.ep-fileName').text(name);
            this.setOption('slides', this.gOptions.slides, true);
          }
          else {
          	var listView = $.tmpl('de_epages.externalcontent.contentsliderview-list', {
              name: name,
              fullpath: fullpath
            });

          	// Translation.
          	listView.find('.ep-sprite[title]').each(function () {
            	this.title = globalDict.translate(this.title);
              this.title = moduleDict.translate(this.title);
            });

            this.elements.slidesList.append(listView).sortable('refresh');
            this.gOptions.slides.push({
              name: name,
              fullpath: fullpath
            });
            this.setOption('slides', this.gOptions.slides, true);
          }
        },
        cropImage: function (file, index) {
          var self = this,
              fullpath = file.get('fullpath'),
              height = file.get('height'),
              width = file.get('width'),
              setting = 'toosmallxy';

          // If any of these three attributes are falsy, we lay back and do nothing.
          if (!(fullpath && height && width)) {
            return;
          }

          if (height === self.defaultSize.height.toString() && width === self.defaultSize.width.toString()) {
            ep.ajax({
              type: 'POST',
              dataType: 'json',
              data: {
                ViewAction: 'JSONViewResponse',
                ChangeAction: 'JSONUploadContentSliderImage',
                ObjectID: ep.config.siteId,
                Path: fullpath
              }
            }).done(function (data) {
              if (data.Errors) {
                return;
              }
              self.addImage(data.filename, data.fullpath, index);
            });
            return;
          }
          self.cropImageSetting = setting;

          // Quick and dirty fix for preview context problem.
          var topDeEpages = window.de_epages,
              topEp = window.ep;

          try {
            topDeEpages = top.de_epages;
            topEp = top.ep;
          }
          catch (error) {}

          self.cropImageDialog = topDeEpages('<div>').externalcontentUiCropslidedialog({
            url: fullpath,
            dialogOptions: $.extend({}, self.cropDialogOptions, {
              close: function (event, ui) {
                self.cropImageDialog.externalcontentUiCropslidedialog('destroy');
              },
              buttons: [{
                text: globalDict.translate('{Apply}'),
                click: $.proxy(function () {
                  ep.ajax({
                    type: 'POST',
                    dataType: 'json',
                    data: {
                      ViewAction: 'JSONViewResponse',
                      ChangeAction: 'JSONUploadContentSliderImage',
                      ObjectID: ep.config.siteId,
                      Path: fullpath,
                      DataURL: self.cropImageDialog.externalcontentUiCropslidedialog('getFabricCanvas').toDataURL()
                    }
                  }).done(function (data) {
                    if (data.Errors) {
                      return;
                    }
                    self.addImage(data.filename, data.fullpath, index);
                  });
                  topEp(self.cropImageDialog).uiDialog('close');

                }, self)
              }]
            })
          });
        }
      });
//#JSCOVERAGE_ENDIF
      return view;
    })(jQuery, Backbone, GadgetViewPrototype),

    googleplus: (function ($, Backbone, GadgetViewPrototype) {
      var view = GadgetViewPrototype.extend({
        alias: 'googleplus',
        events: GadgetViewPrototype.events,
        renderPreview: function () {
          var self = this,
              preview = self.elements.preview,
              parent = preview.parent(),
              gadgetPath = 'externalcontentGadget' + self.alias.charAt(0).toUpperCase() + self.alias.slice(1);
          window.setTimeout(function () {
            preview.remove();
            preview = self.elements.preview = de_epages('<div class="PreviewNode">').appendTo(parent);
            preview[gadgetPath](self.gOptions);
          }, 1);
        }
      });

      return view;
    })(jQuery, Backbone, GadgetViewPrototype),
    rss: (function ($, Backbone, GadgetViewPrototype) {
      var view = GadgetViewPrototype.extend({
        alias: 'rss',
        events: $.extend({}, GadgetViewPrototype.events, {
          'keyup input[name="url"]': 'onUrlKeypress'
        }),
        getAPI: function (callback) {
          // Load Google Loader API.
          $.ajax({
            url: 'https://www.google.com/jsapi',
            dataType: 'script',
            cache: true // We would not want to load the same *script* twice.
          }).done(function () {
            // Load Google Feed API.
            google.load('feeds', '1', {
              callback: callback ||
              function () {}
            });
          });
        },
//#JSCOVERAGE_IF 0
        getFeed: function (url) {
          this.getAPI($.proxy(function () {
            var self = this,
                feed = new google.feeds.Feed(url);
            self.elements.verifyingUrl.removeClass('fail-icon success-icon').addClass('wait-icon');
            feed.load(function (result) {
              if (result.error) {
                self.elements.verifyingUrl.removeClass('wait-icon success-icon').addClass('fail-icon');
              }
              else {
                self.elements.verifyingUrl.removeClass('wait-icon fail-icon').addClass('success-icon');
                self.setOption('url', url, true);
              }
            });
          }, this));
        },
        onUrlKeypress: function (e) {
          var self = this;
          clearTimeout(de_epages.externalcontent.gadgets.verifyUrlTimeout);
          // Wait for user to type in (hopefully valid) feed url.
          de_epages.externalcontent.gadgets.verifyUrlTimeout = window.setTimeout(function () {
            self.getFeed(e.target.value);
          }, 500);
        }
//#JSCOVERAGE_ENDIF
      });

      return view;
    })(jQuery, Backbone, GadgetViewPrototype),
    translation: (function ($, Backbone, GadgetViewPrototype) {
      var view = GadgetViewPrototype.extend({
        alias: 'translation',
        events: GadgetViewPrototype.events,
        renderPreview: function () {
          var self = this,
              preview = self.elements.preview;
          preview.children().remove();
          preview.append('<img src="' + ep.config.storeRoot + '/BO/icons/gadget_translation_' + self.gOptions.service + '.png" />');
        }
      });

      return view;
    })(jQuery, Backbone, GadgetViewPrototype),
    externalvideo: (function ($, Backbone, GadgetViewPrototype) {
      var view = GadgetViewPrototype.extend({
        regexToProvider: {
          'youtu': 'youtube',
          'myvideo': 'myvideo',
          'vimeo': 'vimeo',
          'dailymotion': 'dailymotion',
          'wideo': 'wideofr',
          'metacafe': 'metacafe'
        },
        providerRegex: /youtu|myvideo|vimeo|dailymotion|wideo|metacafe/,
        providerToIdRegex: {
          'youtube': /v=([\w-]+)|be\/([\w-]+)|embed\/([\w-]+)/,
          'myvideo': /watch\/([\w-]+)/,
          'vimeo': /vimeo.com\/?[video]*?\/([\w-]+)/,
          'dailymotion': /video\/([\w-]+)/,
          'wideofr': /wideo\.fr\/video\/([\w-]+)/,
          'metacafe': /watch\/([\w-]+\/[\w-]+)/
        },
        getVideoTypeAndId: function (urlembedString) {
          var self = this,
              videoProvider = self.providerRegex.exec(urlembedString);

          if (videoProvider && videoProvider.length && (videoProvider = videoProvider[0]) && (videoProvider = self.regexToProvider[videoProvider])) {
            var video = self.providerToIdRegex[videoProvider].exec(urlembedString);
            if (video && video.length) {
              var i = 1;
              if (videoProvider === 'youtube') {
                while (video[i] == undefined && i < video.length - 1) {
                  i++;
                }
              }
              return {
                'provider': videoProvider,
                'id': video[i]
              };
            }
          }
        },
        alias: 'externalvideo',
        events: {
          'submit form': 'onSubmit',
          'change input[type!="radio"][name!="video"]': 'onInput',
          'change input[name="video"]': 'onVideoInput'
        },
        onVideoInput: function (e) {
          var self = this,
              target = e.target,
              typeAndId = self.getVideoTypeAndId(target.value);
          self.elements.verifyingUrl.removeClass('fail-icon success-icon');
          if (typeAndId) {
            self.setOptions(typeAndId);
            self.renderPreview();
            self.elements.verifyingUrl.addClass('success-icon');
          }
          else {
            self.setOptions({
              'type': '',
              'id': ''
            });
            self.renderPreview();
            self.elements.verifyingUrl.addClass('fail-icon');
          }
        }
      });
      return view;
    })(jQuery, Backbone, GadgetViewPrototype),
    maps: (function ($, Backbone, GadgetViewPrototype, epConfig, ep) {
      var view = GadgetViewPrototype.extend({
        onRadio: function (e) {
          var target = e.target,
              self = this;
          if (target.value !== self.gOptions[target.name]) {
            if (target.value !== 'klicktel') {
//#JSCOVERAGE_IF 0
              // Set *self.gOptions* according to newly chosen type (i.e. *target.value*) and
              // show fieldset with name *target.value*, hide the others.
              self.gOptions = {};
              self.gOptions[target.name] = target.value;
              self.$el.find('fieldset').not('[name="' + target.name + '"], [name="' + target.value + '"]').attr('disabled', true).addClass('HideElement');
              self.$el.find('fieldset[name="' + target.value + '"]').attr('disabled', false).removeClass('HideElement');
              // Set remaining *self.gOptions* according to fieldset shown.
              self.$el.find('fieldset[name="' + target.value + '"] :input').not('[name="address"]').filter(function () {
                return this.value !== '';
              }).trigger('change', true);
              self.renderPreview();
//#JSCOVERAGE_ENDIF
            }
            else {
              // Set *self.gOptions* according to newly chosen type (i.e. *target.value*) and
              // show fieldset with name *target.value*, hide the others.
              self.gOptions = {};
              self.gOptions[target.name] = target.value;
              self.$el.find('fieldset').not('[name="' + target.name + '"], [name="' + target.value + '"]').attr('disabled', true).addClass('HideElement');
              self.$el.find('fieldset[name="' + target.value + '"]').attr('disabled', false).removeClass('HideElement');
              // Set remaining *self.gOptions* according to fieldset shown.
              self.$el.find('fieldset[name="' + target.value + '"] :input').trigger('change', true);
              self.renderPreview();
            }
          }
        },
        addressToLatLng: function (address, noPreview) {
          var self = this;
          self.geocoder.geocode({
            address: address
          }, $.proxy(function (result, status) {
            var self = this;
            if (status === 'OK' && result && result[0] && result[0].geometry && result[0].geometry.location) {
              delete self.gOptions.address;
              delete self.gOptions.center;
              self.setOptions({
                lat: result[0].geometry.location.lat(),
                lng: result[0].geometry.location.lng()
              });
              if (!noPreview) {
                self.renderPreview();
              }
            }
          }, self));
        },
        renderPreview: function () {
          var self = this,
              preview = self.elements.preview,
              parent = preview.parent(),
              gadgetPath = 'externalcontentGadget' + self.alias.charAt(0).toUpperCase() + self.alias.slice(1);
          if (preview.data('de_epages' + '-' + gadgetPath)) {
            preview[gadgetPath]('destroy');
          }
          preview.remove();
          preview = self.elements.preview = de_epages('<div class="PreviewNode">').appendTo(parent);
          preview[gadgetPath]($.extend({}, self.gOptions, {
            callback: function (type, map, marker) {
              self.map = map;
              self.marker = marker;
              switch (type) {
              case 'google':
                marker.setDraggable(true);
                google.maps.event.addListener(marker, 'dragend', $.proxy(function () {
                  var position = self.marker.getPosition(),
                      center = self.map.getCenter();

                  self.setOptions({
                    lat: position.lat(),
                    lng: position.lng(),
                    center: {
                      lat: center.lat(),
                      lng: center.lng()
                    }
                  });

                }, self));

                google.maps.event.addListener(map, 'dragend', $.proxy(function () {
                  var center = self.map.getCenter();

                  self.setOptions({
                    center: {
                      lat: center.lat(),
                      lng: center.lng()
                    }
                  });

                }, self));

                google.maps.event.addListener(map, 'zoom_changed', $.proxy(function () {
                  self.setOptions({
                    zoom: self.map.getZoom()
                  });

                }, self));

                google.maps.event.addListener(map, 'maptypeid_changed', $.proxy(function () {
                  self.setOptions({
                    mapType: self.map.getMapTypeId()
                  });

                }, self));

                break;
//#JSCOVERAGE_IF 0
              case 'bing':
                self.map.AttachEvent('onchangeview', $.proxy(function (e) {
                  var center = this.map.GetCenter();

                  this.setOptions({
                    mapType: e.mapStyle,
                    zoom: e.zoomLevel,
                    center: {
                      lat: center.Latitude,
                      lng: center.Longitude
                    }
                  });

                }, self));
//#JSCOVERAGE_ENDIF
                break;
              }
            }
          }));
        },
        render: function () {
          var self = this,
              types = [],
              i;

          if (self.differTypes) {
            // Merge *self.gOptions* in *types* for templating. (See below.)
            for (i = 0; i < self.gDefaults.length; i++) {
              var defaults = self.gDefaults[i];
              types.push(defaults.type === self.gOptions.type ? self.gOptions : defaults);
            }
          }

          $.ready({
            plugin: ['ep.core', self.plugin],
            tmpl: self.template
          }, function () {
            var renderedTemplate = $.tmpl(self.template, self.differTypes ? {
              'types': types,
              'gOptions': self.gOptions
            } : self.gOptions).dictParse('ep.dict', true).dictParse('de_epages.externalcontent.gadgets', true);
            renderedTemplate.find(':input').each(function () {
              if (this.placeholder) {
                this.placeholder = globalDict.translate(this.placeholder);
                this.placeholder = moduleDict.translate(this.placeholder);
              }
              ep(this).uiInput();
            });
            self.$el.append(renderedTemplate);
            self.elements = renderedTemplate.tmplItem('elements'); // preview element z.B.
            self.elements.preview = de_epages(self.elements.preview);

            ep.ajax({
              url: 'https://www.google.com/jsapi',
              dataType: 'script',
              cache: true // We would not want to load the same *script* twice.
            }).done(function () {
              google.load('maps', '3', {
                other_params: 'libraries=places&sensor=false',
                callback: function () {
                  self.geocoder = new google.maps.Geocoder();
                  self.$('input[name="address"]').each(function () {
                    var autocomplete = new google.maps.places.Autocomplete(this);
                    google.maps.event.addListener(autocomplete, 'place_changed', function () {
//#JSCOVERAGE_IF 0
                      var place = this.getPlace();
                      if (place.geometry && place.geometry.location) {
                        delete self.gOptions.address;
                        delete self.gOptions.center;
                        self.setOptions({
                          lat: place.geometry.location.lat(),
                          lng: place.geometry.location.lng()
                        });
                        self.renderPreview();
                      }
                      self.addressToLatLng(place.name);
                    });
//#JSCOVERAGE_ENDIF
                  });
                  if (!self.gOptions.lat || !self.gOptions.lng) {
                    if (epConfig.shopAddress) {
                      self.addressToLatLng(epConfig.shopAddress, true);
                      self.$('input[name="address"]').each(function () {
                        this.value = epConfig.shopAddress;
                      });
                    }
                  }
                  self.renderPreview();
                }
              });
            });
          });
          return this;
        },
        alias: 'maps',
        events: GadgetViewPrototype.events
      });
      return view;
    })(jQuery, Backbone, GadgetViewPrototype, ep.config, ep)
  });

  $.extend($.scope('de_epages.externalcontent'), {
    gadgets: function (options) {
      if (gadgetDialogInstance) {
        gadgetDialogInstance.openDialog(options);
      }
      else {
        gadgetDialogInstance = new GadgetsDialog(options);
      }
    }
  });

});