/*
 * @copyright		© Copyright 2006-2012, epages GmbH, All Rights Reserved.
 *
 * @module			ep.ui.slidebox
 *
 * @revision		$Revision: 1.1.2.1.2.1 $
 */

jQuery.plugin( "ep.ui.slidebox", ["ep.ui.core", "jQuery.event.special.touch", "jquery.pp.animate", "jQuery.ui.widget"], function( $, undefined ){

	var currentPageItem = "ep-uiSlidebox-item";

	$.widget( "ep.ui.Slidebox", {
		options: {
			view	: 2,
			arrow	: true,
			padding	: true,
			speed	: 300
		},

		_create: function(){
			var self = this,
				o    = self.options,
				elem = self.elem = self.element;

			elem
				.addClass("ep-uiSlidebox")
				.toggleClass("ep-uiSlidebox-noarrow", !o.arrow)
				.on("touchdrag.uiSlidebox", function( event ){
					if( event.dragState == "start" ){
						var diff = Math.abs(event.dragDeltaX) - Math.abs(event.dragDeltaY);

						self.state.locked = diff < -2 ? "y" : diff > 2 ? "x" : "";
					}

					if( event.dragState != "start" && self.state.locked == "x" ){
						event.preventDefault();
					}

					if( self.state.locked == "x" ){
						self.list.css("left", (parseInt(self.list.css("left")) + (event.dragDeltaX * 100 / self.elem.width())) + "%");
					}

					if (event.dragDeltaX === 0) {
						self.state.direction = self.state.direction || 1;
					}
					else {
						self.state.direction = event.dragDeltaX > 0 ? 1 : -1;
					}

					if( event.dragState == "stop" ){
						var relativeLeft = Math.min(0,parseInt(self.list.css("left"))) + (30 * self.state.direction),
							index = Math.round(Math.abs(relativeLeft)/100),
							page = index + 1;

						self.state.locked = "";

						self.goTo(page);
					}
				});

			self.list = elem.children().first()
				.addClass("ep-uiSlidebox-list");

			self.prev = $('<a href="javascript:void(\'Previous\');" class="ep-uiSlidebox-prev"><span class="ep-uiSlidebox-arrow">&lt;</span></a>')
				.on("click.uiSlidebox", function(event){
					event.preventDefault();
					self.goTo("prev");
				})
				.appendTo(elem);

			self.next = $('<a href="javascript:void(\'Next\');" class="ep-uiSlidebox-next"><span class="ep-uiSlidebox-arrow">&gt;</span></a>')
				.on("click.uiSlidebox", function(event){
					event.preventDefault();
					self.goTo("next");
				})
				.appendTo(elem);

			self.counts = {
				page	: 1,
				pages	: 1,
				items	: 1
			};

			self.sizes = {
				nav		: 10,
				list	: 80,
				item	: 100
			};

			self.state = {
				direction: 1,
				locked: ""
			};

			self._setOptions(o);
		},

		_init: function(){
			var self	= this,
				view	= self.options.view,
				items	= self.items = self.list.children(),
				counts	= self.counts,
				sizes	= self.sizes;

			// Count of list items
			counts.items 	= items.length;
			// Count of pages in list
			counts.pages 	= Math.ceil(counts.items / view);
			// Calculate navigation field width (min 8%)
			sizes.nav		= Math.max(8,100 * 0.25 / (view+0.5));
			// Set padding to list if set
			sizes.padding 	= self.options.padding ? sizes.nav : 0;
			// Calculate whole list width
			sizes.list		= counts.pages * (100 - (2*sizes.padding));
			// Calculate width of single list item
			sizes.item		= 100 / (counts.pages * view);

			self.list
				.css({
					"left"		: "0%",
					"padding"	: "0 " + sizes.padding + "%",
					"width"		: sizes.list + "%",
					"max-width"	: sizes.list + "%"				// Android 2.x doesn't support value 'none' for property 'max-width'
				});

			items
				.css({
					"width"		: sizes.item + "%"
				});

			self.prev
				.css({
					"width"		: sizes.nav + "%"
				});

			self.next
				.css({
					"width"		: sizes.nav + "%"
				});

			self.goTo(counts.page);
		},

		_setOption: function( key, value ) {
			var self = this,
				reinit = false;

			if( key === "arrow" ){
				self.elem.toggleClass("ep-uiSlidebox-noarrow", !value);
			}

			this.Inherited(arguments);

			self._init();
		},

		goTo: function( i ){
			var self = this,
				page = self.counts.page,
				view = self.options.view,
				callback = function(){
					self.items
						.slice( (slice = (page-1) * view), slice + view)
						.addClass(currentPageItem);

					self.prev.toggle(page != 1);
					self.next.toggle(page != self.counts.pages);

					self.elem.trigger("changeslide", {page: page, pages: self.counts.pages});
				},
				getLeft,
				setLeft,
				slice;

			if( i == 'next' ){
				page++;
			}
			else if( i == 'prev' ){
				page--;
			}
			else{
				page = i;
			}

			self.items
				.removeClass(currentPageItem);

			page = Math.max(1,Math.min(self.counts.pages,page));

			getLeft = self.list.css("left").replace(/px/,"%"); // Firefox returns 'px', but value is '%'

			setLeft = ( (100 - (2*self.sizes.padding)) * (page-1) * -1 ) + "%";


			if( setLeft != getLeft ){
				self.counts.page = page;

				self.list
					.stop(true)
					.animate({
							left: setLeft
						},
						self.option.speed,
						"linear",
						callback
					);
			}
			else{
				callback();
			}

			return this;
		},

		destroy: function(){
			var self = this;

			self.elem
				.off(".uiSlidebox")
				.removeClass("ep-uiSlidebox");

			self.list
				.removeClass("ep-uiSlidebox-list");

			self.items
				.removeClass(currentPageItem);

			self.prev
				.remove();

			self.next
				.remove();

			self.Inherited(arguments);
		}
	});

});