/*
 * @copyright		© Copyright 2006-2012, epages GmbH, All Rights Reserved.
 *
 * @module			de_epages.externalcontent.ui.cropslidedialog
 *
 * @revision		$$
 */

jQuery.plugin('de_epages.externalcontent.ui.cropslidedialog', {
  plugin: ['de_epages.core', 'ep.dict', 'jQuery.ui.widget', 'jQuery.tmpl', 'fabric', 'ep.ui.dialog'],
  tmpl: ['de_epages.externalcontent.ui.cropslidedialog'],
  dict: ['de_epages.externalcontent.gadgets']
}, function ($) {

/*
 * @dictionary de_epages.externalcontent.gadgets
 * {RepositionYourImage}
 * {ImageTooSmall}
*/

  var moduleDict = ep.dict('de_epages.externalcontent.gadgets');

  // The actual widget.
  $.widget('de_epages.externalcontent.ui.Cropslidedialog', {

    options: {
      'width': 1050,
      'height': 350,
      'imageOptions': {},
      'dialogOptions': {},
      'canvasOptions': {},
      'url': '',
      'headerInfoText': ''
    },

    _create: function () {
      var self = this,
          o = self.options;

      $.extend(self, $.tmplItem($.tmpl('de_epages.externalcontent.ui.cropslidedialog', o), 'elements'));
      ep(self.element.append(self.header, self.canvas)).uiDialog(o.dialogOptions);
      self.fabricCanvas = new fabric.Canvas(self.canvas[0], o.canvasOptions);
      fabric.Image.fromURL(o.url, self.Proxy('_onImageLoad'), o.imageOptions);
    },

    _setOption: function (key, value) {
      if (key === 'headerInfoText') {
        this.header.html(value);
      }
      return this.Inherited(arguments);
    },

    getFabricCanvas: function () {
      return this.fabricCanvas;
    },

    getFabricImage: function () {
      return this.fabricImage;
    },

    _onImageLoad: function (image) {
      var self = this,
          o = self.options;

			// Store *image* for later access through *getFabricImage* method.
      self.fabricImage = image;
      // We do not allow scaling and rotating.
      image.hasControls = false;
      // Do not draw borders on image.
      image.hasBorders = false;

      // Now let's see which *setting* we have here.
      if (image.width < (o.width + 1)) {
        if (image.height > (o.height)) {
          image.lockMovementX = true;
          self._setOption('headerInfoText', moduleDict.translate('{RepositionYourImage}'));
        }
        else {
          image.lockMovementX = image.lockMovementY = true;
          self.fabricCanvas.moveCursor = self.fabricCanvas.hoverCursor = self.fabricCanvas.defaultCursor;
          self._setOption('headerInfoText', moduleDict.translate('{ImageTooSmall}'));
        }
      }
      else {
        if (image.height < (o.height + 1)) {
          image.lockMovementY = true;
          self._setOption('headerInfoText', moduleDict.translate('{RepositionYourImage}'));
        }
        else {
          if (image.width > (3 * image.height - 1)) {
            image.scaleToHeight(o.height);
            image.lockMovementY = true;
            self._setOption('headerInfoText', moduleDict.translate('{RepositionYourImage}'));
          }
          else {
            image.scaleToWidth(o.width);
						image.lockMovementX = true;
            self._setOption('headerInfoText', moduleDict.translate('{RepositionYourImage}'));
          }
        }
      }

			// Center image.
      image.set(self.fabricCanvas.getCenter());

			// Add image to *canvas*.
      self.fabricCanvas.add(image);
    },

    destroy: function () {
      var self = this;
      // Remove all traces of the widget.
      self.canvas.remove();
      ep(self.element).uiDialog('destroy');
      return self.Inherited(arguments);
    }

  });

});